/*
Arduino-MAX30100 oximetry / heart rate integrated sensor library
Copyright (C) 2016  OXullo Intersecans <x@brainrapers.org>
*/


#include <Wire.h>
#include "MAX30100_PulseOximeter.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>

namespace uuid {
  static const char *SERVICE_UUID = "0000FFE0-0000-1000-8000-00805F9B34FB";
  static const char *CHARACTERISTIC_UUID = "0000FFE1-0000-1000-8000-00805F9B34FB";
} // namespace uuid

// forward declare parser for incoming messages
void parseGuioMsg(const String &msg);

// custom handling of server callbacks
class CustomBLEServerCallbacks: public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      Serial.println("Connected!");
    };
    void onDisconnect(BLEServer* pServer) {
      Serial.println("Disconnected!");
    }
};

// custom handling of characteristic callbacks
class CustomBLECharacteristicCallbacks: public BLECharacteristicCallbacks {
  void onWrite(BLECharacteristic *pCharacteristic) {
    String msg = pCharacteristic->getValue();
      // parse message string
    parseGuioMsg(String(msg.c_str()));
  }      
};

// global ptr
BLECharacteristic *pCharacteristic;

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels
#define REPORTING_PERIOD_MS     1000
#define OLED_RESET     21// Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// PulseOximeter is the higher level interface to the sensor
//  * beat detection reporting
//  * heart rate calculation
//  * SpO2 (oxidation level) calculation
PulseOximeter pox;

uint32_t tsLastReport = 0;
char buffer[100];
volatile bool heartbeatFlag = false;
// Callback (registered below) fired when a pulse is detected
void onBeatDetected()
{
    heartbeatFlag=true;
}

void setup()
{
    Serial.begin(115200);
    delay(2000);
    Serial.println("Startup");
    // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  display.display();
  
  // create BLE device
  BLEDevice::init("HeartRate");
  // create server and register callback
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new CustomBLEServerCallbacks());
  // create service
  BLEService *pService = pServer->createService(uuid::SERVICE_UUID);
  // crate characteristic, register callback and add descriptor
  pCharacteristic = pService->createCharacteristic(uuid::CHARACTERISTIC_UUID,
                                                   BLECharacteristic::PROPERTY_NOTIFY |
                                                   BLECharacteristic::PROPERTY_INDICATE |
                                                   BLECharacteristic::PROPERTY_WRITE |
                                                   BLECharacteristic::PROPERTY_READ);
  pCharacteristic->setCallbacks(new CustomBLECharacteristicCallbacks());
  pCharacteristic->addDescriptor(new BLE2902());

  // start the service and start advertising
  pService->start();
  pServer->getAdvertising()->start();


  // Clear the buffer
  display.clearDisplay();
  display.display();
  display.setTextSize(2); // Draw 2X-scale text
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
    Serial.print("Initializing pulse oximeter..");

    // Initialize the PulseOximeter instance
    // Failures are generally due to an improper I2C wiring, missing power supply
    // or wrong target chip
    if (!pox.begin()) {
        Serial.println("FAILED");
        for(;;);
    } else {
        Serial.println("SUCCESS");
    }

    // The default current for the IR LED is 50mA and it could be changed
    //   by uncommenting the following line. Check MAX30100_Registers.h for all the
    //   available options.
    // pox.setIRLedCurrent(MAX30100_LED_CURR_7_6MA);

    // Register a callback for the beat detection
    pox.setOnBeatDetectedCallback(onBeatDetected);
}

void loop()
{
    // Make sure to call update as fast as possible
    pox.update();
    if (heartbeatFlag) {
      Serial.println("Beat!");
      heartbeatFlag=false;
    }
    // Asynchronously dump heart rate and oxidation levels to the serial
    // For both, a value of 0 means "invalid"
    if (millis() - tsLastReport > REPORTING_PERIOD_MS) {
        Serial.print("Heart rate= ");
        float hr;
        hr=pox.getHeartRate();
        int h;
        h= (int) (0.5+ hr);
        Serial.print(h);
        Serial.print(" BPM  ");
        display.clearDisplay();
        display.setCursor(0, 0);
        display.print("HR=");
        display.print(h);
        display.print(" BPM");
        sprintf(buffer,"@lb1 TXT:\"%3d\"\r\n",h);
        sendMsg(buffer);
        float s;
        s= pox.getSpO2();
        h=(int) (0.5 + s); 
        display.setCursor(0,18);
        display.print("SPO2= ");
        display.print(h);
        display.print("%"); 
        display.display();
        sprintf(buffer,"@lb3 TXT:\"%3d\"\r\n",h);
        sendMsg(buffer);
        Serial.print("SpO2= ");
        Serial.print(s);
        Serial.println(" %");
        digitalWrite(15,LOW);  // heartbeat LED off
        tsLastReport = millis();
    }
}

void sendMsg(const String &msg) {
  pCharacteristic->setValue(String(msg.c_str()));
  pCharacteristic->notify();
  delay(50);
}

void parseGuioMsg(const String &msg) {
  if(msg.startsWith("@init")) {
    Serial.println("GUI-O app is requesting INITIALIZATION!");

    // clear screen and set background
    sendMsg("@cls\r\n");
    sendMsg("@guis BGC:#FFFFFF\r\n");
    delay(100);

    // initialize the GUI
  sendMsg("|LB UID:lb0 X:50 Y:10 FSZ:6 TXT:\"Heart Rate\"\r\n");
  sendMsg("|LB UID:lb1 X:50 Y:20 FSZ:6 TXT:\"----\"\r\n");
  sendMsg("|LB UID:lb2 X:50 Y:30 FGC:#FF0274 FSZ:6 TXT:\"SPO2\"\r\n");
  sendMsg("|LB UID:lb3 X:50 Y:40 FSZ:6 TXT:\"---\"\r\n");
  }
 }  


